package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.ids.InputExpressionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.InputId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import org.apache.commons.lang3.Validate;

public class VDmnInputColumnImpl extends VDmnColumnImpl implements VDmnInputColumn {

  private InputExpressionId inputExpressionId;

  private VDmnInputColumnImpl() {
    super();
  }

  public static Builder getBuilder() {
    return new VDmnInputColumnImpl().new Builder();
  }

  @Override
  public InputExpressionId getInputExpressionId() {
    return inputExpressionId;
  }

  public class Builder extends DmnElementBuilder<VDmnInputColumnImpl> {

    public Builder withInputId(InputId inputId) {
      value.id = inputId;
      return this;
    }

    public Builder withDecisionTable(VDmnDecisionTable decisionTable) {
      value.dmnDecisionTable = decisionTable;
      return this;
    }

    public Builder withInputExpressionId(InputExpressionId inputExpressionId) {
      value.inputExpressionId = inputExpressionId;
      return this;
    }

    public Builder withTypeRef(VTypeRef typeRef) {
      value.typeRef = typeRef;
      return this;
    }

    public Builder addValue(VDmnInputValue dmnValue) {
      value.values.add(Validate.notNull(dmnValue));
      return this;
    }

    public Builder addValueFromBuilder(VDmnInputValueImpl.Builder dmnValueBuilder) {
      addValue(Validate.notNull(dmnValueBuilder.getUnbuildValue()));
      dmnValueBuilder.withDmnInputColumn(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.typeRef);
      Validate.notNull(value.inputExpressionId);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
