package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.ids.OutputId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VTypeRef;
import org.apache.commons.lang3.Validate;

public class VDmnOutputColumnImpl extends VDmnColumnImpl implements VDmnOutputColumn {

  private VDmnOutputColumnImpl() {
    super();
  }

  public static Builder getBuilder() {
    return new VDmnOutputColumnImpl().new Builder();
  }

  public class Builder extends DmnElementBuilder<VDmnOutputColumnImpl> {

    public Builder withOutputId(OutputId outputId) {
      value.id = outputId;
      return this;
    }

    public Builder withDecisionTable(VDmnDecisionTable decisionTable) {
      value.dmnDecisionTable = decisionTable;
      return this;
    }

    public Builder withTypeRef(VTypeRef typeRef) {
      value.typeRef = typeRef;
      return this;
    }

    public Builder addValue(VDmnOutputValue dmnValue) {
      value.values.add(Validate.notNull(dmnValue));
      return this;
    }

    public Builder addValueFromBuilder(VDmnOutputValueImpl.Builder dmnValueBuilder) {
      addValue(Validate.notNull(dmnValueBuilder.getUnbuildValue()));
      dmnValueBuilder.withDmnOutputColumn(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.typeRef);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
